program read_ramses
  !---------------------------------------------------------------------
  ! Ce programme lit un fichier particule de RAMSES.
  ! R. Teyssier
  ! Saclay, le 28/02/2006
  !---------------------------------------------------------------------
  implicit none
  integer::ndim,npart,ngrid,idim,icpu,ipos,ncpu,i
  integer::ncpu2,npart2,ndim2
  real(kind=8)::mtot
  real,dimension(:,:),allocatable::xp,vp
  real,dimension(:),allocatable::mp,tmp

  character*5::nchar,ncharcpu
  character*50::nomfich,repository
  logical::ok

  write(*,*)'Enter output directory'
  read(*,'(a)')repository

  !-----------------------------------------------
  ! Lecture du fichier particules au format RAMSES
  !-----------------------------------------------
  ipos=INDEX(repository,'output_')
  nchar=repository(ipos+7:ipos+13)
  nomfich=TRIM(repository)//'/part_'//TRIM(nchar)//'.out00001'
  inquire(file=nomfich, exist=ok) ! verify input file 
  if ( .not. ok ) then
     print *,TRIM(nomfich)//' not found.'
     stop
  endif

  open(unit=1,file=nomfich,status='old',form='unformatted')
  read(1)ncpu
  read(1)ndim
  close(1)

  npart=0
  do icpu=1,ncpu
  call title(icpu,ncharcpu)
  nomfich=TRIM(repository)//'/part_'//TRIM(nchar)//'.out'//TRIM(ncharcpu)
  write(*,*)'Reading file '//TRIM(nomfich)
  open(unit=1,file=nomfich,status='old',form='unformatted')
  read(1)ncpu2
  read(1)ndim2
  read(1)npart2
  close(1)
  npart=npart+npart2
  end do

  write(*,*)'Found ',npart,' particles'
  write(*,*)'Reading positions and masses...'

  allocate(xp(1:npart,1:ndim))
  allocate(vp(1:npart,1:ndim))
  allocate(mp(1:npart))

  npart=0
  do icpu=1,ncpu
  call title(icpu,ncharcpu)
  nomfich=TRIM(repository)//'/part_'//TRIM(nchar)//'.out'//TRIM(ncharcpu)
  open(unit=1,file=nomfich,status='old',form='unformatted')
  read(1)ncpu2
  read(1)ndim2
  read(1)npart2
  allocate(tmp(1:npart2))
  do idim=1,ndim
     read(1)tmp
     xp(npart+1:npart+npart2,idim)=tmp
  end do
  do idim=1,ndim
     read(1)tmp
     vp(npart+1:npart+npart2,idim)=tmp
  end do
  read(1)tmp
  mp(npart+1:npart+npart2)=tmp
  close(1)
  npart=npart+npart2
  deallocate(tmp)
  end do

  mtot=0.0d0
  do i=1,npart
     mtot=mtot+mp(i)
  enddo
  write(*,*)'mtot=',mtot

end program read_ramses

subroutine title(n,nchar)
  integer::n
  character*5::nchar

  character*1::nchar1
  character*2::nchar2
  character*3::nchar3
  character*4::nchar4
  character*5::nchar5


  if(n.ge.10000)then
     write(nchar5,'(i5)') n
     nchar = nchar5
  elseif(n.ge.1000)then
     write(nchar4,'(i4)') n
     nchar = '0'//nchar4
  elseif(n.ge.100)then
     write(nchar3,'(i3)') n
     nchar = '00'//nchar3
  elseif(n.ge.10)then
     write(nchar2,'(i2)') n
     nchar = '000'//nchar2
  else
     write(nchar1,'(i1)') n
     nchar = '0000'//nchar1
  endif
end subroutine title

